@@ -118,7 +118,7 @@ class options(ranaModule):
 
 
 
-
+    # network
     self.addOption("Network", "network",
 #      [("off","No use of network"),
       [("minimal", "Only for important data"),
@@ -126,6 +126,9 @@ class options(ranaModule):
        "network",
        "full")
 
+    # gps
+    self.addBoolOption("GPS", "GPSEnabled", "GPS", True, "gpsd:checkGPSEnabled")
+
 #    self.addOption("Network", "threadedDownload",
 ##      [("off","No use of network"),
 #      [("True", "Use threads for download"),
@@ -133,11 +136,11 @@ class options(ranaModule):
 #       "network",
 #       "on")
 
-    self.addBoolOption("Logging", "logging", "logging", True)
-    options = []
-    for i in (1,2,5,10,20,40,60):
-      options.append((i, "%d sec" % i))
-    self.addOption("Frequency", "log_period", options, "logging", 2)
+#    self.addBoolOption("Logging", "logging", "logging", True)
+#    options = []
+#    for i in (1,2,5,10,20,40,60):
+#      options.append((i, "%d sec" % i))
+#    self.addOption("Frequency", "log_period", options, "logging", 2)
 
 #    self.addBoolOption("Vector maps", "vmap", "map", True)
 
@@ -218,6 +221,48 @@ class options(ranaModule):
 
 
     # * navigation
+
+    # in the first string: first one goes to espeak, the seccond part goes to Google
+    directionsLanguages =[('ca ca', 'Catalan'),
+                          ('zh-yue zh-TW', 'Chinese(Cantonese)'),
+                          ('zh zh-CN', 'Chinese(Mandarin)'),
+                          ('hr hr', 'Croatian'),
+                          ('cs cs', 'Czech'),
+                          ('nl nl', 'Dutch'),
+                          ('en en', 'English'),
+                          ('fi fi', 'Finnish'),
+                          ('fr fr', 'French'),
+                          ('de de', 'German'),
+                          ('el el', 'Greek'),
+                          ('hi hi', 'Hindi'),
+                          ('hu hu', 'Hungarian'),
+                          ('id id', 'Indonesian'),
+                          ('it it', 'Italian'),
+                          ('lv lv', 'Latvian'),
+                          ('no no', 'Norwegian'),
+                          ('pl pl', 'Polish'),
+                          ('pt pt-BR', 'Portuguese(Brazil)'),
+                          ('pt-pt pt-PT', 'Portuguese(European)'),
+                          ('ro ro', 'Romanian'),
+                          ('ru ru', 'Russian'),
+                          ('sr sr', 'Serbian'),
+                          ('sk sk', 'Slovak'),
+                          ('es es', 'Spanish'),
+                          ('ta ta', 'Tamil'),
+                          ('tr tr', 'Turkish'),
+                          ('vi vi', 'Vietnamese')]
+
+    self.addOption("Language for directions", "directionsLanguage",directionsLanguages,
+       "Navigation",
+       "en en") # TODO: use locale for default language ?
+
+    self.addOption("Autostart navigation", "autostartNavigation",
+      [('disabled',"Disabled"),
+       ('first',"From first point"),
+       ('closest',"From closest point")],
+       "Navigation",
+       'disabled')
+
     self.addOption("Point reached distance", "pointReachedDistance",
       [(10,"10 m"),
        (20,"20 m"),
@@ -246,8 +291,6 @@ class options(ranaModule):
        "POI",
        "dump")
 
-
-
     # Add all our categories to the "options" menu
     self.menuModule = self.m.get("menu", None)
     if(self.menuModule):
