@@ -59,7 +59,7 @@ class turnByTurn(ranaModule):
     elif message == 'reroute':
       # 1. say rerouting is in progress
       message = "rerouting"
-      self.espeakSay(message, 0)
+      self.espeakSay(message, 0, "en") # make sure rerouting said with english voice
       time.sleep(2) #TODO: improve this
       # 2. get a new route from current position to destination
       self.sendMessage("ms:route:reroute:fromPosToDest")
@@ -122,7 +122,6 @@ class turnByTurn(ranaModule):
       lon = currentStep['Point']['coordinates'][0]
 
       self.currentStepIndicator = proj.ll2xy(lat, lon)
-      print self.currentStepIndicator
 #      cr.set_source_rgb(1, 0, 0)
 #      cr.set_line_width(4)
 #      cr.arc(pointX, pointY, 12, 0, 2.0 * math.pi)
@@ -176,7 +175,7 @@ class turnByTurn(ranaModule):
           clickHandler.registerXYWH(bx, by , bw, bh, action)
 
 
-  def espeakSay(self, plaintextMessage, distanceMeters):
+  def espeakSay(self, plaintextMessage, distanceMeters, forceLanguageCode=False):
     """say routing messages through espeak"""
     units = self.m.get('units', None)
     if units:
@@ -184,11 +183,17 @@ class turnByTurn(ranaModule):
         distString = ""
       else:
         distString = units.km2CurrentUnitString(distanceMeters/1000.0, 1, False)
-        distString = 'in <emphasis level="strong">'+ distString + '</emphasis><br>'
+        distString = '<p xml:lang="en">in <emphasis level="strong">'+ distString + '</emphasis></p><br>'
+        # TODO: language specific distance strings
       output = distString + plaintextMessage
       print "saying: %s" % output
-      print plaintextMessage
-      self.espaekProcess = subprocess.Popen(['espeak', '-s 120','-m','"%s"' % output])
+      if forceLanguageCode:
+        espeakLanguageCode = forceLanguageCode
+      else:
+        # the espeak language code is the fisrt part of this whitespace delimited string
+        espeakLanguageCode = self.get('directionsLanguage', 'en en').split(" ")[0]
+      languageParam = '-v%s' % espeakLanguageCode
+      self.espaekProcess = subprocess.Popen(['espeak', languageParam ,'-s 120','-m','"%s"' % output])
 
   def getStartingStep(self, which='first'):
     if self.steps:
@@ -208,12 +213,6 @@ class turnByTurn(ranaModule):
         lon2 = step['Point']['coordinates'][0]
         step['currentDistance'] = geo.distance(lat1,lon1,lat2,lon2)*1000 # km to m
       closestStep = sorted(tempSteps, key=lambda x: x['currentDistance'])[0]
-      for step in self.steps:
-        print step['currentDistance']
-        print step['id']
-
-      print "adasdasd"
-      print closestStep['id']
       return closestStep
 
   def getCurrentStep(self):
@@ -230,13 +229,15 @@ class turnByTurn(ranaModule):
 
   def switchToNextStep(self):
     """switch to next step and clean up"""
-    print "switching to next step"
     maxIndex = len(self.steps) - 1
     nextIndex = self.currentStepIndex + 1
     if nextIndex <= maxIndex:
       self.currentStepIndex = nextIndex
       self.espeakFirstTrigger = False
       self.espeakSecondTrigger = False
+      print "switching to next step"
+    else:
+      print "last step reached"
 
   def enabled(self):
     """return True if enabled, flase othervise"""
@@ -247,18 +248,18 @@ class turnByTurn(ranaModule):
 
   def startTBT(self):
     # start Turn-by-turn navigation
-    self.sendMessage('notification:use at own risk, watch for cliffs, etc.#2')
-
     m = self.m.get('route', None)
     if m:
-     dirs = m.getCurrentDirections()
-     if dirs: # is the route nonempty ?
-       route = dirs['Directions']['Routes'][0]
-       self.steps = []
-       for step in route['Steps']:
-         step['currentDistance'] = None # add the currentDistance key
-         self.steps.append(step)
-       self.steps = dirs['Directions']['Routes'][0]['Steps']
+      dirs = m.getCurrentDirections()
+      if dirs: # is the route nonempty ?
+        print dirs
+        self.sendMessage('notification:use at own risk, watch for cliffs, etc.#2')
+        route = dirs['Directions']['Routes'][0]
+        self.steps = []
+        for step in route['Steps']:
+          step['currentDistance'] = None # add the currentDistance key
+          self.steps.append(step)
+        self.steps = dirs['Directions']['Routes'][0]['Steps']
 
   def stopTBT(self):
     # stop Turn-by-turn navigation
